@@ -128,6 +128,46 @@ void pmap3_getaddr_cb(struct rpc_context *rpc, int status, void *data, void *pri
 	client->is_finished = 1;
 }
 
+void pmap2_set_cb(struct rpc_context *rpc, int status, void *data, void *private_data)
+{
+	struct client *client = private_data;
+	uint32_t res = *(uint32_t *)data;
+
+	if (status == RPC_STATUS_ERROR) {
+		printf("PORTMAP2/SET call failed with \"%s\"\n", (char *)data);
+		exit(10);
+	}
+	if (status != RPC_STATUS_SUCCESS) {
+		printf("PORTMAP2/SET call failed, status:%d\n", status);
+		exit(10);
+	}
+
+	printf("PORTMAP2/SET:\n");
+	printf("	Res:%d\n", res);
+
+	client->is_finished = 1;
+}
+
+void pmap2_unset_cb(struct rpc_context *rpc, int status, void *data, void *private_data)
+{
+	struct client *client = private_data;
+	uint32_t res = *(uint32_t *)data;
+
+	if (status == RPC_STATUS_ERROR) {
+		printf("PORTMAP2/UNSET call failed with \"%s\"\n", (char *)data);
+		exit(10);
+	}
+	if (status != RPC_STATUS_SUCCESS) {
+		printf("PORTMAP2/UNSET call failed, status:%d\n", status);
+		exit(10);
+	}
+
+	printf("PORTMAP2/UNSET:\n");
+	printf("	Res:%d\n", res);
+
+	client->is_finished = 1;
+}
+
 void pmap3_set_cb(struct rpc_context *rpc, int status, void *data, void *private_data)
 {
 	struct client *client = private_data;
@@ -329,6 +369,8 @@ int main(int argc _U_, char *argv[] _U_)
 	int null2 = 0;
 	int dump2 = 0;
 	int null3 = 0;
+	int set2 = 0;
+	int unset2 = 0;
 	int set3 = 0;
 	int unset3 = 0;
 	int getaddr3 = 0;
@@ -337,6 +379,8 @@ int main(int argc _U_, char *argv[] _U_)
 	int u2t3 = 0;
 	int command_found = 0;
 
+	int set2prog, set2vers, set2prot, set2port;
+	int unset2prog, unset2vers, unset2prot, unset2port;
 	int set3prog, set3vers;
 	char *set3netid, *set3addr, *set3owner;
 	int unset3prog, unset3vers;
@@ -365,6 +409,20 @@ int main(int argc _U_, char *argv[] _U_)
 		} else if (!strcmp(argv[i], "null2")) {
 			null2 = 1;
 			command_found++;
+		} else if (!strcmp(argv[i], "set2")) {
+			set2 = 1;
+			set2prog = atoi(argv[++i]);
+			set2vers = atoi(argv[++i]);
+			set2prot = atoi(argv[++i]);
+			set2port = atoi(argv[++i]);
+			command_found++;
+		} else if (!strcmp(argv[i], "unset2")) {
+			unset2 = 1;
+			unset2prog = atoi(argv[++i]);
+			unset2vers = atoi(argv[++i]);
+			unset2prot = atoi(argv[++i]);
+			unset2port = atoi(argv[++i]);
+			command_found++;
 		} else if (!strcmp(argv[i], "dump3")) {
 			dump3 = 1;
 			command_found++;
@@ -465,6 +523,20 @@ int main(int argc _U_, char *argv[] _U_)
 		}
 		wait_until_finished(rpc, &client);
 	}
+	if (set2) {
+		if (rpc_pmap2_set_async(rpc, set2prog, set2vers, set2prot, set2port, pmap2_set_cb, &client) != 0) {
+			printf("Failed to send SET2 request\n");
+			exit(10);
+		}
+		wait_until_finished(rpc, &client);
+	}
+	if (unset2) {
+		if (rpc_pmap2_unset_async(rpc, unset2prog, unset2vers, unset2prot, unset2port, pmap2_unset_cb, &client) != 0) {
+			printf("Failed to send UNSET2 request\n");
+			exit(10);
+		}
+		wait_until_finished(rpc, &client);
+	}
 	if (set3) {
 		struct pmap3_mapping map;
 
