@@ -22,16 +22,31 @@
 #include "config.h"  /* HAVE_SOCKADDR_STORAGE ? */
 #endif
 
+#ifdef HAVE_NET_IF_H
+#include <net/if.h>
+#endif
+
 #ifndef WIN32
 #include <sys/socket.h>  /* struct sockaddr_storage */
 #endif
 
+#if defined(WIN32) && !defined(IFNAMSIZ)
+#define IFNAMSIZ 255
+#endif
+
 #include "libnfs-zdr.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#ifndef MIN
+#define MIN(a, b) (((a) < (b)) ? (a) : (b))
+#endif
+#ifndef MAX
+#define MAX(a, b) (((a) > (b)) ? (a) : (b))
+#endif
+
 #if !defined(HAVE_SOCKADDR_STORAGE) && !defined(WIN32)
 /*
  * RFC 2553: protocol-independent placeholder for socket addresses
@@ -58,7 +73,7 @@ struct sockaddr_storage {
 
 struct rpc_fragment {
 	struct rpc_fragment *next;
-	uint64_t size;
+	uint32_t size;
 	char *data;
 };
 
@@ -75,11 +90,23 @@ struct rpc_queue {
 #define HASHES 1024
 #define NFS_RA_TIMEOUT 5
 #define NFS_MAX_XFER_SIZE (1024 * 1024)
+#define ZDR_ENCODE_OVERHEAD 1024
+#define ZDR_ENCODEBUF_MINSIZE 4096
+
+struct rpc_endpoint {
+        struct rpc_endpoint *next;
+        int program;
+        int version;
+        struct service_proc *procs;
+        int num_procs;
+};
 
 struct rpc_context {
 	uint32_t magic;
 	int fd;
+	int old_fd;
 	int is_connected;
+	int is_nonblocking;
 
 	char *error_string;
 
@@ -89,26 +116,24 @@ struct rpc_context {
 	struct AUTH *auth;
 	uint32_t xid;
 
-	/* buffer used for encoding RPC PDU */
-	char *encodebuf;
-	int encodebuflen;
-
 	struct rpc_queue outqueue;
 	struct sockaddr_storage udp_src;
 	struct rpc_queue waitpdu[HASHES];
+	uint32_t waitpdu_len;
 
 	uint32_t inpos;
+	char rm_buf[4];
 	char *inbuf;
-	uint32_t inbuflen;
 
 	/* special fields for UDP, which can sometimes be BROADCASTed */
 	int is_udp;
-	struct sockaddr *udp_dest;
+	struct sockaddr_storage udp_dest;
 	int is_broadcast;
 
 	/* track the address we connect to so we can auto-reconnect on session failure */
 	struct sockaddr_storage s;
 	int auto_reconnect;
+	int num_retries;
 
 	/* fragment reassembly */
 	struct rpc_fragment *fragments;
@@ -118,7 +143,15 @@ struct rpc_context {
 	int uid;
 	int gid;
 	uint32_t readahead;
+	uint32_t pagecache;
+	uint32_t pagecache_ttl;
 	int debug;
+	int timeout;
+	char ifname[IFNAMSIZ];
+
+        /* Is a server context ? */
+        int is_server_context;
+        struct rpc_endpoint *endpoints;
 };
 
 struct rpc_pdu {
@@ -137,6 +170,11 @@ struct rpc_pdu {
 	zdrproc_t zdr_decode_fn;
 	caddr_t zdr_decode_buf;
 	uint32_t zdr_decode_bufsize;
+
+#define PDU_DISCARD_AFTER_SENDING 0x00000001
+        uint32_t flags;
+
+	time_t timeout;
 };
 
 void rpc_reset_queue(struct rpc_queue *q);
@@ -145,9 +183,10 @@ void rpc_return_to_queue(struct rpc_queue *q, struct rpc_pdu *pdu);
 unsigned int rpc_hash_xid(uint32_t xid);
 
 struct rpc_pdu *rpc_allocate_pdu(struct rpc_context *rpc, int program, int version, int procedure, rpc_cb cb, void *private_data, zdrproc_t zdr_decode_fn, int zdr_bufsize);
+struct rpc_pdu *rpc_allocate_pdu2(struct rpc_context *rpc, int program, int version, int procedure, rpc_cb cb, void *private_data, zdrproc_t zdr_decode_fn, int zdr_bufsize, size_t alloc_hint);
 void rpc_free_pdu(struct rpc_context *rpc, struct rpc_pdu *pdu);
 int rpc_queue_pdu(struct rpc_context *rpc, struct rpc_pdu *pdu);
-int rpc_get_pdu_size(char *buf);
+uint32_t rpc_get_pdu_size(char *buf);
 int rpc_process_pdu(struct rpc_context *rpc, char *buf, int size);
 void rpc_error_all_pdus(struct rpc_context *rpc, const char *error);
 
@@ -179,20 +218,26 @@ int rpc_set_udp_destination(struct rpc_context *rpc, char *addr, int port, int i
 struct rpc_context *rpc_init_udp_context(void);
 struct sockaddr *rpc_get_recv_sockaddr(struct rpc_context *rpc);
 
-void rpc_set_autoreconnect(struct rpc_context *rpc);
-void rpc_unset_autoreconnect(struct rpc_context *rpc);
+void rpc_set_autoreconnect(struct rpc_context *rpc, int num_retries);
+
+void rpc_set_interface(struct rpc_context *rpc, const char *ifname);
 
 void rpc_set_tcp_syncnt(struct rpc_context *rpc, int v);
-void rpc_set_uid(struct rpc_context *rpc, int uid);
-void rpc_set_gid(struct rpc_context *rpc, int gid);
+void rpc_set_pagecache(struct rpc_context *rpc, uint32_t v);
+void rpc_set_pagecache_ttl(struct rpc_context *rpc, uint32_t v);
 void rpc_set_readahead(struct rpc_context *rpc, uint32_t v);
 void rpc_set_debug(struct rpc_context *rpc, int level);
-
-int rpc_add_fragment(struct rpc_context *rpc, char *data, uint64_t size);
+void rpc_set_timeout(struct rpc_context *rpc, int timeout);
+int rpc_get_timeout(struct rpc_context *rpc);
+int rpc_add_fragment(struct rpc_context *rpc, char *data, uint32_t size);
 void rpc_free_all_fragments(struct rpc_context *rpc);
+int rpc_is_udp_socket(struct rpc_context *rpc);
+int rpc_current_time(void);
 
 const struct nfs_fh3 *nfs_get_rootfh(struct nfs_context *nfs);
 
+void *zdr_malloc(ZDR *zdrs, uint32_t size);
+
 #ifdef __cplusplus
 }
 #endif
