@@ -26,7 +26,9 @@
 #include "win32_compat.h"
 #endif
 
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
+#endif
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -48,6 +50,18 @@
 #include "libnfs-raw.h"
 #include "libnfs-private.h"
 
+int rpc_current_time(void)
+{
+#ifdef HAVE_CLOCK_GETTIME
+	struct timespec tp;
+
+	clock_gettime(CLOCK_MONOTONIC_COARSE, &tp);
+	return tp.tv_sec;
+#else
+	return time(NULL);
+#endif
+}
+
 struct rpc_context *rpc_init_context(void)
 {
 	struct rpc_context *rpc;
@@ -62,31 +76,16 @@ struct rpc_context *rpc_init_context(void)
 
 	rpc->magic = RPC_CONTEXT_MAGIC;
 
-	/* Allow NFS_MAX_XFER_SIZE of data (for writes) and some */
-	rpc->encodebuflen = NFS_MAX_XFER_SIZE + 4096;
-	rpc->encodebuf = malloc(rpc->encodebuflen);
-	if (rpc->encodebuf == NULL) {
-		free(rpc);
-		return NULL;
-	}
-
-	rpc->inbuflen = 2 * (NFS_MAX_XFER_SIZE + 4096);
-	rpc->inbuf = malloc(rpc->inbuflen);
-	if (rpc->inbuf == NULL) {
-		free(rpc);
-		return NULL;
-	}
-
  	rpc->auth = authunix_create_default();
 	if (rpc->auth == NULL) {
-		free(rpc->encodebuf);
 		free(rpc);
 		return NULL;
 	}
-	rpc->xid = salt + time(NULL) + (getpid() << 16);
+	rpc->xid = salt + rpc_current_time() + (getpid() << 16);
 	salt += 0x01000000;
 	rpc->fd = -1;
 	rpc->tcp_syncnt = RPC_PARAM_UNDEFINED;
+	rpc->pagecache_ttl = NFS_PAGECACHE_DEFAULT_TTL;
 #if defined(WIN32) || defined(ANDROID)
 	rpc->uid = 65534;
 	rpc->gid = 65534;
@@ -98,15 +97,96 @@ struct rpc_context *rpc_init_context(void)
 	for (i = 0; i < HASHES; i++)
 		rpc_reset_queue(&rpc->waitpdu[i]);
 
+	/* Default is no timeout */
+	rpc->timeout = -1;
+
 	return rpc;
 }
 
+struct rpc_context *rpc_init_server_context(int s)
+{
+	struct rpc_context *rpc;
+
+	rpc = malloc(sizeof(struct rpc_context));
+	if (rpc == NULL) {
+		return NULL;
+	}
+	memset(rpc, 0, sizeof(struct rpc_context));
+
+	rpc->magic = RPC_CONTEXT_MAGIC;
+
+	rpc->is_server_context = 1;
+	rpc->fd = s;
+	rpc->is_connected = 1;
+        rpc->is_udp = rpc_is_udp_socket(rpc);
+	rpc_reset_queue(&rpc->outqueue);
+
+	return rpc;
+}
+
+static uint32_t round_to_power_of_two(uint32_t x) {
+	uint32_t power = 1;
+	while (power < x) {
+		power <<= 1;
+	}
+	return power;
+}
+
+void rpc_set_pagecache(struct rpc_context *rpc, uint32_t v)
+{
+	assert(rpc->magic == RPC_CONTEXT_MAGIC);
+	v = MAX(rpc->pagecache, round_to_power_of_two(v));
+	RPC_LOG(rpc, 2, "pagecache set to %d pages of size %d", v, NFS_BLKSIZE);
+	rpc->pagecache = v;
+}
+
+void rpc_set_pagecache_ttl(struct rpc_context *rpc, uint32_t v) {
+	if (v) {
+		RPC_LOG(rpc, 2, "set pagecache ttl to %d seconds\n", v);
+	} else {
+		RPC_LOG(rpc, 2, "set pagecache ttl to infinite");
+	}
+	rpc->pagecache_ttl = v;
+}
+
 void rpc_set_readahead(struct rpc_context *rpc, uint32_t v)
 {
+	uint32_t min_pagecache;
+
+	assert(rpc->magic == RPC_CONTEXT_MAGIC);
+	if (v) {
+		v = MAX(NFS_BLKSIZE, round_to_power_of_two(v));
+	}
+	RPC_LOG(rpc, 2, "readahead set to %d byte", v);
+	rpc->readahead = v;
+	min_pagecache = (2 * v) / NFS_BLKSIZE;
+	if (rpc->pagecache < min_pagecache) {
+		/* current pagecache implementation needs a pagecache bigger
+		 * than the readahead size to avoid collisions */
+		rpc_set_pagecache(rpc, min_pagecache);
+	}
+}
+
+#ifdef HAVE_SO_BINDTODEVICE
+void rpc_set_interface(struct rpc_context *rpc, const char *ifname)
+{
+	/*
+	 * This only copies the interface information into the RPC
+	 * structure.  It doesn't stop whatever interface is being used. The
+	 * connection needs to be restarted for that happen. In other words,
+	 * set this before you connect.
+	 */
 	assert(rpc->magic == RPC_CONTEXT_MAGIC);
 
-	rpc->readahead = v;
+	if (ifname) {
+		/*
+		 * Allow at one-less character just-in-case IFNAMSIZ for
+		 * the defined platform does not include the NUL-terminator.
+		 */
+		strncpy(rpc->ifname, ifname, sizeof(rpc->ifname) - 1);
+	}
 }
+#endif
 
 void rpc_set_debug(struct rpc_context *rpc, int level)
 {
@@ -168,7 +248,7 @@ void rpc_set_error(struct rpc_context *rpc, const char *error_string, ...)
 	vsnprintf(rpc->error_string, 1024, error_string, ap);
         va_end(ap);
 
-	fprintf(stderr, "libnfs error: %s\n", rpc->error_string);
+	RPC_LOG(rpc, 1, "error: %s", rpc->error_string);
 
 	if (old_error_string != NULL) {
 		free(old_error_string);
@@ -182,31 +262,51 @@ char *rpc_get_error(struct rpc_context *rpc)
 	return rpc->error_string;
 }
 
-void rpc_error_all_pdus(struct rpc_context *rpc, const char *error)
+static void rpc_purge_all_pdus(struct rpc_context *rpc, int status, const char *error)
 {
+	struct rpc_queue outqueue;
 	struct rpc_pdu *pdu;
-	unsigned int i;
+	int i;
 
 	assert(rpc->magic == RPC_CONTEXT_MAGIC);
 
-	while ((pdu = rpc->outqueue.head) != NULL) {
-          pdu->cb(rpc, RPC_STATUS_ERROR, (void *)error, pdu->private_data);
-		rpc->outqueue.head = pdu->next;
+	/* Remove all entries from each queue before cancellation to prevent
+	 * the callbacks manipulating entries that are about to be removed.
+	 *
+	 * This code assumes that the callbacks will not enqueue any new
+	 * pdus when called.
+	 */
+
+	outqueue = rpc->outqueue;
+
+	rpc_reset_queue(&rpc->outqueue);
+	while ((pdu = outqueue.head) != NULL) {
+		outqueue.head = pdu->next;
+		pdu->next = NULL;
+		pdu->cb(rpc, status, (void *) error, pdu->private_data);
 		rpc_free_pdu(rpc, pdu);
 	}
-	rpc->outqueue.tail = NULL;
 
 	for (i = 0; i < HASHES; i++) {
-		struct rpc_queue *q = &rpc->waitpdu[i];
+		struct rpc_queue waitqueue = rpc->waitpdu[i];
 
-		while((pdu = q->head) != NULL) {
-                  pdu->cb(rpc, RPC_STATUS_ERROR, (void *)error,
-                          pdu->private_data);
-			q->head = pdu->next;
+		rpc_reset_queue(&rpc->waitpdu[i]);
+		while((pdu = waitqueue.head) != NULL) {
+			waitqueue.head = pdu->next;
+			pdu->next = NULL;
+			pdu->cb(rpc, status, (void *) error, pdu->private_data);
 			rpc_free_pdu(rpc, pdu);
 		}
-		q->tail = NULL;
 	}
+
+	assert(!rpc->outqueue.head);
+	for (i = 0; i < HASHES; i++)
+		assert(!rpc->waitpdu[i].head);
+}
+
+void rpc_error_all_pdus(struct rpc_context *rpc, const char *error)
+{
+	rpc_purge_all_pdus(rpc, RPC_STATUS_ERROR, error);
 }
 
 static void rpc_free_fragment(struct rpc_fragment *fragment)
@@ -229,7 +329,7 @@ void rpc_free_all_fragments(struct rpc_context *rpc)
 	}
 }
 
-int rpc_add_fragment(struct rpc_context *rpc, char *data, uint64_t size)
+int rpc_add_fragment(struct rpc_context *rpc, char *data, uint32_t size)
 {
 	struct rpc_fragment *fragment;
 
@@ -254,43 +354,72 @@ int rpc_add_fragment(struct rpc_context *rpc, char *data, uint64_t size)
 
 void rpc_destroy_context(struct rpc_context *rpc)
 {
-	struct rpc_pdu *pdu;
-	unsigned int i;
-
 	assert(rpc->magic == RPC_CONTEXT_MAGIC);
 
-	while((pdu = rpc->outqueue.head) != NULL) {
-		pdu->cb(rpc, RPC_STATUS_CANCEL, NULL, pdu->private_data);
-		LIBNFS_LIST_REMOVE(&rpc->outqueue.head, pdu);
-		rpc_free_pdu(rpc, pdu);
-	}
-
-	for (i = 0; i < HASHES; i++) {
-		struct rpc_queue *q = &rpc->waitpdu[i];
-
-		while((pdu = q->head) != NULL) {
-			pdu->cb(rpc, RPC_STATUS_CANCEL, NULL, pdu->private_data);
-			LIBNFS_LIST_REMOVE(&q->head, pdu);
-			rpc_free_pdu(rpc, pdu);
-		}
-	}
+	rpc_purge_all_pdus(rpc, RPC_STATUS_CANCEL, NULL);
 
 	rpc_free_all_fragments(rpc);
 
-	auth_destroy(rpc->auth);
-	rpc->auth =NULL;
+        if (rpc->auth) {
+                auth_destroy(rpc->auth);
+                rpc->auth =NULL;
+        }
 
 	if (rpc->fd != -1) {
  		close(rpc->fd);
 	}
 
-	free(rpc->encodebuf);
+	if (rpc->error_string != NULL) {
+		free(rpc->error_string);
+		rpc->error_string = NULL;
+	}
+
 	free(rpc->inbuf);
-	free(rpc->error_string);
-	free(rpc->udp_dest);
+	rpc->inbuf = NULL;
 
 	rpc->magic = 0;
 	free(rpc);
 }
 
+void rpc_set_timeout(struct rpc_context *rpc, int timeout)
+{
+	assert(rpc->magic == RPC_CONTEXT_MAGIC);
 
+	rpc->timeout = timeout;
+}
+
+int rpc_get_timeout(struct rpc_context *rpc)
+{
+	assert(rpc->magic == RPC_CONTEXT_MAGIC);
+
+	return rpc->timeout;
+}
+
+int rpc_register_service(struct rpc_context *rpc, int program, int version,
+                         struct service_proc *procs, int num_procs)
+{
+        struct rpc_endpoint *endpoint;
+
+        assert(rpc->magic == RPC_CONTEXT_MAGIC);
+
+        if (!rpc->is_server_context) {
+		rpc_set_error(rpc, "Not a server context.");
+                return -1;
+        }
+
+        endpoint = malloc(sizeof(*endpoint));
+        if (endpoint == NULL) {
+		rpc_set_error(rpc, "Out of memory: Failed to allocate endpoint "
+                              "structure");
+                return -1;
+        }
+
+        endpoint->program = program;
+        endpoint->version = version;
+        endpoint->procs = procs;
+        endpoint->num_procs = num_procs;
+        endpoint->next = rpc->endpoints;
+        rpc->endpoints = endpoint;
+
+        return 0;
+}
