@@ -88,7 +88,7 @@ int nfsstat3_to_errno(int error)
 	case NFS3ERR_NAMETOOLONG: return -ENAMETOOLONG; break;
 	case NFS3ERR_NOTEMPTY:    return -ENOTEMPTY; break;
 	case NFS3ERR_DQUOT:       return -ERANGE; break;
-	case NFS3ERR_STALE:       return -EIO; break;
+	case NFS3ERR_STALE:       return -ESTALE; break;
 	case NFS3ERR_REMOTE:      return -EIO; break;
 	case NFS3ERR_BADHANDLE:   return -EIO; break;
 	case NFS3ERR_NOT_SYNC:    return -EIO; break;
@@ -118,7 +118,6 @@ int rpc_nfs3_null_async(struct rpc_context *rpc, rpc_cb cb, void *private_data)
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/NULL call");
-		rpc_free_pdu(rpc, pdu);
 		return -2;
 	}
 
@@ -148,7 +147,6 @@ int rpc_nfs3_getattr_async(struct rpc_context *rpc, rpc_cb cb, struct GETATTR3ar
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/GETATTR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -184,7 +182,6 @@ int rpc_nfs3_pathconf_async(struct rpc_context *rpc, rpc_cb cb, struct PATHCONF3
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/PATHCONF call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -220,7 +217,6 @@ int rpc_nfs3_lookup_async(struct rpc_context *rpc, rpc_cb cb, struct LOOKUP3args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/LOOKUP call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -257,7 +253,6 @@ int rpc_nfs3_access_async(struct rpc_context *rpc, rpc_cb cb, struct ACCESS3args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/ACCESS call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -294,7 +289,6 @@ int rpc_nfs3_read_async(struct rpc_context *rpc, rpc_cb cb, struct READ3args *ar
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/READ call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -332,7 +326,6 @@ int rpc_nfs3_write_async(struct rpc_context *rpc, rpc_cb cb, struct WRITE3args *
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/WRITE call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -373,7 +366,6 @@ int rpc_nfs3_commit_async(struct rpc_context *rpc, rpc_cb cb, struct COMMIT3args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/COMMIT call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -411,7 +403,6 @@ int rpc_nfs3_setattr_async(struct rpc_context *rpc, rpc_cb cb, SETATTR3args *arg
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/SETATTR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -441,7 +432,6 @@ int rpc_nfs3_mkdir_async(struct rpc_context *rpc, rpc_cb cb, MKDIR3args *args, v
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/MKDIR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -471,7 +461,6 @@ int rpc_nfs3_rmdir_async(struct rpc_context *rpc, rpc_cb cb, struct RMDIR3args *
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/RMDIR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -508,7 +497,6 @@ int rpc_nfs3_create_async(struct rpc_context *rpc, rpc_cb cb, CREATE3args *args,
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/CREATE call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -538,7 +526,6 @@ int rpc_nfs3_mknod_async(struct rpc_context *rpc, rpc_cb cb, struct MKNOD3args *
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/MKNOD call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -604,7 +591,6 @@ int rpc_nfs3_remove_async(struct rpc_context *rpc, rpc_cb cb, struct REMOVE3args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/REMOVE call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -641,7 +627,6 @@ int rpc_nfs3_readdir_async(struct rpc_context *rpc, rpc_cb cb, struct READDIR3ar
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/READDIR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -680,7 +665,6 @@ int rpc_nfs3_readdirplus_async(struct rpc_context *rpc, rpc_cb cb, struct READDI
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/READDIRPLUS call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -720,7 +704,6 @@ int rpc_nfs3_fsstat_async(struct rpc_context *rpc, rpc_cb cb, struct FSSTAT3args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/FSSTAT call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -756,7 +739,6 @@ int rpc_nfs3_fsinfo_async(struct rpc_context *rpc, rpc_cb cb, struct FSINFO3args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/FSINFO call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -792,7 +774,6 @@ int rpc_nfs3_readlink_async(struct rpc_context *rpc, rpc_cb cb, READLINK3args *a
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/READLINK call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -822,7 +803,6 @@ int rpc_nfs3_symlink_async(struct rpc_context *rpc, rpc_cb cb, SYMLINK3args *arg
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/SYMLINK call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -852,7 +832,6 @@ int rpc_nfs3_rename_async(struct rpc_context *rpc, rpc_cb cb, struct RENAME3args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/RENAME call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -892,7 +871,6 @@ int rpc_nfs3_link_async(struct rpc_context *rpc, rpc_cb cb, struct LINK3args *ar
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS3/LINK call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -928,7 +906,6 @@ int rpc_nfs2_null_async(struct rpc_context *rpc, rpc_cb cb, void *private_data)
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/NULL call");
-		rpc_free_pdu(rpc, pdu);
 		return -2;
 	}
 
@@ -953,7 +930,6 @@ int rpc_nfs2_getattr_async(struct rpc_context *rpc, rpc_cb cb, struct GETATTR2ar
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/GETATTR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -978,7 +954,6 @@ int rpc_nfs2_setattr_async(struct rpc_context *rpc, rpc_cb cb, SETATTR2args *arg
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/SETATTR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1003,7 +978,6 @@ int rpc_nfs2_lookup_async(struct rpc_context *rpc, rpc_cb cb, struct LOOKUP2args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/LOOKUP call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1028,7 +1002,6 @@ int rpc_nfs2_readlink_async(struct rpc_context *rpc, rpc_cb cb, READLINK2args *a
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/READLINK call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1053,7 +1026,6 @@ int rpc_nfs2_read_async(struct rpc_context *rpc, rpc_cb cb, struct READ2args *ar
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/READ call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1078,7 +1050,6 @@ int rpc_nfs2_write_async(struct rpc_context *rpc, rpc_cb cb, struct WRITE2args *
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/WRITE call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1103,7 +1074,6 @@ int rpc_nfs2_create_async(struct rpc_context *rpc, rpc_cb cb, CREATE2args *args,
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/CREATE call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1128,7 +1098,6 @@ int rpc_nfs2_remove_async(struct rpc_context *rpc, rpc_cb cb, struct REMOVE2args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/REMOVE call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1153,7 +1122,6 @@ int rpc_nfs2_rename_async(struct rpc_context *rpc, rpc_cb cb, struct RENAME2args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/RENAME call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1178,7 +1146,6 @@ int rpc_nfs2_link_async(struct rpc_context *rpc, rpc_cb cb, LINK2args *args, voi
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/LINK call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1203,7 +1170,6 @@ int rpc_nfs2_symlink_async(struct rpc_context *rpc, rpc_cb cb, SYMLINK2args *arg
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/SYMLINK call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1228,7 +1194,6 @@ int rpc_nfs2_mkdir_async(struct rpc_context *rpc, rpc_cb cb, MKDIR2args *args, v
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/MKDIR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1253,7 +1218,6 @@ int rpc_nfs2_rmdir_async(struct rpc_context *rpc, rpc_cb cb, struct RMDIR2args *
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/RMDIR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1278,7 +1242,6 @@ int rpc_nfs2_readdir_async(struct rpc_context *rpc, rpc_cb cb, struct READDIR2ar
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/READDIR call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
@@ -1303,7 +1266,6 @@ int rpc_nfs2_statfs_async(struct rpc_context *rpc, rpc_cb cb, struct STATFS2args
 
 	if (rpc_queue_pdu(rpc, pdu) != 0) {
 		rpc_set_error(rpc, "Out of memory. Failed to queue pdu for NFS2/STATFS call");
-		rpc_free_pdu(rpc, pdu);
 		return -3;
 	}
 
